from .building import Building

class MatrixLab(Building):
    pass

    def get_size():
        return Vector(5.0, 5.0)

    def get_offset():
        return Vector(2.5, 2.5)

class SelfEvolutionLab(Building):
    pass